<?php
/**
 * Created by PhpStorm.
 * User: francismararaujo
 * Date: 23/10/14
 * Time: 17:07
 */

namespace TecDiet\Services;

use Doctrine\ORM\EntityManager;
use TecDiet\Util\Validation;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class AppointmentsService implements Queries {

    const SELECTED = 'S';
    const UNSELECTED = 'N';

    protected $entityManager;
    protected $connection;
    protected $session;

    public function __construct(EntityManager $entityManager, $session) {

        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
        $this->session = $session;

    }

    private function insertAppointment($appointment) {
        $appointment['status'] = self::SELECTED;
        $sth = $this->connection->prepare(self::INSERT_APPOINTMENT);
        $appointment['height'] = Validation::sanitazeNumber($appointment['height'], false);
        $appointment['weight'] = Validation::sanitazeNumber($appointment['weight'], false);
        $sth->execute(
            array(
                'CDPACIENTE'    => $appointment['id'],
                'DTCONSPACI'    => $appointment['date'],
                'CDFILIAL'      => $appointment['unit'],
                'CDOPERADOR'    => $appointment['nutritionistId'],
                'NRORG'         => $appointment['organization'],
                'IDATIVO'       => $appointment['status'],
                'CDNIASNUTR'    => $appointment['nutritionalStatusId'],
                'CDEQUIMED'     => $appointment['medicalTeamId'],
                'CDSERPADDIE'   => $appointment['healthInsuranceId'],
                'CDDIETPAD'     => $appointment['dietId'],
                'VRPESOPACI'    => $appointment['weight'],
                'VRALTURAPACI'  => $appointment['height']
            )
        );

    }

    private function updateAppointment($appointment) {

        $appointment['height'] = Validation::sanitazeNumber($appointment['height'], false);
        $appointment['weight'] = Validation::sanitazeNumber($appointment['weight'], false);
        $sth = $this->connection->prepare(self::UPDATE_APPOINTMENT);

        $sth->bindParam(':CDNIASNUTR', $appointment['nutritionalStatusId']);
        $sth->bindParam(':CDEQUIMED', $appointment['medicalTeamId']);
        $sth->bindParam(':CDSERPADDIE', $appointment['healthInsuranceId']);
        $sth->bindParam(':CDDIETPAD', $appointment['dietId']);
        $sth->bindParam(':VRPESOPACI', $appointment['weight']);
        $sth->bindParam(':VRALTURAPACI', $appointment['height']);
        $sth->bindParam(':OLD_CDPACIENTE', $appointment['id']);
        $sth->bindParam(':OLD_DTCONSPACI', $appointment['date']);
        $sth->bindParam(':OLD_CDFILIAL', $appointment['unit']);

        $sth->execute();

    }

    private function insertDieteticPrescription($prescription) {

        $sth = $this->connection->prepare(self::INSERT_DIETETIC_PRESCRIPTION);
        
        $sth->bindParam(':CDPACIENTE', $prescription['id']);
        $sth->bindParam(':DTPRESDIET', $prescription['date']);
        $sth->bindParam(':CDDIETPAD', $prescription['dietId']);
        $sth->bindParam(':DSPRESDIET', $prescription['prescription']);
        $sth->bindParam(':DSDADOCLIN', $prescription['clinical']);
        $sth->bindParam(':DSPRESMEDDIE', $prescription['medicalPrescription']);
        $sth->bindParam(':CDFILIAL', $prescription['unit']);
        $sth->bindParam(':CDNIASNUTR', $prescription['nutritionalStatusId']);
        $sth->bindParam(':CDLEITO', $prescription['bed']);

        $sth->execute();

    }

    private function isExistAppointment($appointment) {
        if (!isset($appointment['appointmentDate'])){
            $appointment['appointmentDate'] = $appointment['date'];
        }
        $date = \DateTime::createFromFormat('d/m/Y H:i:s', $appointment['appointmentDate'], new \DateTimeZone('America/Sao_Paulo'));
        
        $sth = $this->connection->prepare(self::APPOINTMENTS_NOW);
        $sth->execute(array(
            'CDPACIENTE' => $appointment['id'],
            'CDFILIAL' => $appointment['unit'],
            'DTCONSPACI' => $date->format('d/m/Y H:i:s')
        ));
        return $sth->fetchAll(\PDO::FETCH_ASSOC);
    }

    public function save($appointments) {
        $erroToSaveAppointments = array(
            'APPOINTMENT' => array(),
            'ERROR' => array(),
            'OPERATION' => array()
        );
        foreach ($appointments as $appointment) {
            $errorToVerifiedAppointment = false;
            try {
                $existAppointment = $this->isExistAppointment($appointment);
            } catch (\Exception $e) {
                $errorToVerifiedAppointment = true;
                array_push($erroToSaveAppointments['APPOINTMENT'], $appointment);
                array_push($erroToSaveAppointments['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                array_push($erroToSaveAppointments['OPERATION'], 'SELECT');
            }
            if(!$errorToVerifiedAppointment){
                if($existAppointment) {
                    try {
                        $this->updateAppointment($appointment);
                    } catch (\Exception $e) {
                        array_push($erroToSaveAppointments['APPOINTMENT'], $appointment);
                        array_push($erroToSaveAppointments['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                        array_push($erroToSaveAppointments['OPERATION'], 'UPDATE');
                    }
                } else {
                    try {
                        $this->insertAppointment($appointment);
                    } catch (\Exception $e) {
                        array_push($erroToSaveAppointments['APPOINTMENT'], $appointment);
                        array_push($erroToSaveAppointments['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                        array_push($erroToSaveAppointments['OPERATION'], 'INSERT');
                    }
                }
            }
        }
        return $erroToSaveAppointments;
    }

    private function updateAnthEvaluationMethod($anthEvaluationMethod) {
        $sth = $this->connection->prepare(self::UPDATE_ANTH_EVALUATION_METHOD);
        if(!empty($anthEvaluationMethod['selectedRow'])) {
            $anthEvaluationMethod['selected'] = self::SELECTED;
        } else {
            $anthEvaluationMethod['selected'] = self::UNSELECTED;
        }

        $sth->bindParam(':IDTABELAAV', $anthEvaluationMethod['selected']);
        $sth->bindParam(':OLD_CDFILIAL', $anthEvaluationMethod['unit']);
        $sth->bindParam(':OLD_CDPACIENTE', $anthEvaluationMethod['id']);
        $sth->bindParam(':OLD_DTCONSPACI', $anthEvaluationMethod['appointmentDate']);
        $sth->bindParam(':OLD_CDTBMETOAVAL', $anthEvaluationMethod['method']);
        $sth->bindParam(':CDTABELACO', $anthEvaluationMethod['reference']);

        $sth->execute();
    }

    private function insertAnthEvaluationMethod($anthEvaluationMethod) {
        $sth = $this->connection->prepare(self::INSERT_ANTH_EVALUATION_METHOD);
        if(!empty($anthEvaluationMethod['__isSelected'])){
            $anthEvaluationMethod['selected'] = self::SELECTED;
        } else {
            $anthEvaluationMethod['selected'] = self::UNSELECTED;
        }
        
        $date = \DateTime::createFromFormat('d/m/Y H:i:s', $anthEvaluationMethod['appointmentDate'], new \DateTimeZone('America/Sao_Paulo'));
        
        $sth->execute(array(
            "CDFILIAL" => $anthEvaluationMethod['unit'],
            "CDPACIENTE" => $anthEvaluationMethod['id'],
            "DTCONSPACI" => $date->format('d/m/Y H:i:s'),
            "IDTABELAAV" => $anthEvaluationMethod['selected'],
            "CDTABELACO" => $anthEvaluationMethod['reference'],
            "CDTBMETOAVAL" => $anthEvaluationMethod['method']
        ));

    }

    private function getMethodByPatient($anthEvaluationMethod) {
        if(empty($anthEvaluationMethod['method'])){
            $anthEvaluationMethod['method'] = $anthEvaluationMethod['methodEvaluationId'];
        }
        $sth = $this->connection->prepare(self::ANTHROPOMETRIC_EVALUATION_METHOD);
        $sth->execute(array(
            'CDPACIENTE' => $anthEvaluationMethod['id'],
            'CDFILIAL' => $anthEvaluationMethod['unit'],
            'DTCONSPACI' => $anthEvaluationMethod['appointmentDate'],
            'CDTBMETOAVAL' => $anthEvaluationMethod['method']
        ));

        return $sth->fetch(\PDO::FETCH_ASSOC);

    }

    public function saveAnthEvaluationMethods($anthEvaluationMethods) {
        $erroToSaveAnthEvaluation = array(
            'ANTHROPOMETRIC' => array(),
            'ERROR' => array(),
            'OPERATION' => array()
        );
        foreach ($anthEvaluationMethods as $anthEvaluationMethod) {
            try{
                $method = $this->getMethodByPatient($anthEvaluationMethod);
            }catch(\Exception $e){
                array_push($erroToSaveAnthEvaluation['ANTHROPOMETRIC'], $anthEvaluationMethod);
                array_push($erroToSaveAnthEvaluation['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                array_push($erroToSaveAnthEvaluation['OPERATION'], 'SELECT');
            }
            if($method) {
                try {
                    $this->updateAnthEvaluationMethod($anthEvaluationMethod);
                } catch (\Exception $e) {
                    array_push($erroToSaveAnthEvaluation['ANTHROPOMETRIC'], $anthEvaluationMethod);
                    array_push($erroToSaveAnthEvaluation['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                    array_push($erroToSaveAnthEvaluation['OPERATION'], 'UPDATE');
                }
            }else {
                try {
                $this->insertAnthEvaluationMethod($anthEvaluationMethod);
                } catch (\Exception $e) {
                    array_push($erroToSaveAnthEvaluation['ANTHROPOMETRIC'], $anthEvaluationMethod);
                    array_push($erroToSaveAnthEvaluation['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                    array_push($erroToSaveAnthEvaluation['OPERATION'], 'INSERT');
                }
            }

        }
        return $erroToSaveAnthEvaluation;
    }

    public function insertDataInfoEvaluation($dataInfoEvaluation) {

        $sth = $this->connection->prepare(self::INSERT_DATA_INFO_EVALUATION);

        $dataInfoEvaluation['value'] = Validation::sanitazeNumber($dataInfoEvaluation['value'], false);
        
        $sth->execute(
            array(
                "CDFILIAL" => $dataInfoEvaluation['unit'],
                "CDPACIENTE" => $dataInfoEvaluation['id'],
                "DTCONSPACI" => $dataInfoEvaluation['appointmentDate'],
                "CDTBMETOAVAL" => $dataInfoEvaluation['methodEvaluationId'],
                "CDTIPODADO" => $dataInfoEvaluation['dataTypeEvaluationId'],
                "DSINFOAVAL" => $dataInfoEvaluation['value']
            )
        );

    }

    public function updateDataInfoEvaluation($dataInfoEvaluation) {

        $sth = $this->connection->prepare(self::UPDATE_DATA_INFO_EVALUATION);
        
        $dataInfoEvaluation['value'] = Validation::sanitazeNumber($dataInfoEvaluation['value'], false);
        
        $sth->execute(
            array(
                "DSINFOAVAL" => $dataInfoEvaluation['value'],
                "OLD_CDFILIAL" => $dataInfoEvaluation['unit'],
                "OLD_CDPACIENTE" => $dataInfoEvaluation['id'],
                "OLD_DTCONSPACI" => $dataInfoEvaluation['appointmentDate'],
                "OLD_CDTBMETOAVAL" => $dataInfoEvaluation['methodEvaluationId'],
                "OLD_CDTIPODADO" => $dataInfoEvaluation['dataTypeEvaluationId']
            )
        );

    }

    private function getDataInfoByPatient($dataInfoEvaluation) {

        $sth = $this->connection->prepare(self::ANTHROPOMETRIC_EVALUATION_DATA_INFO);

        $sth->execute(array(
            'CDPACIENTE' => $dataInfoEvaluation['id'],
            'CDFILIAL' => $dataInfoEvaluation['unit'],
            'DTCONSPACI' => $dataInfoEvaluation['appointmentDate'],
            'CDTBMETOAVAL' => $dataInfoEvaluation['methodEvaluationId'],
            'CDTIPODADO' => $dataInfoEvaluation['dataTypeEvaluationId']
        ));

        return $sth->fetch(\PDO::FETCH_ASSOC);

    }

    public function saveDataInfoEvaluations($dataInfoEvaluations) {

        foreach ($dataInfoEvaluations as $dataInfoEvaluation) {

            $dataInfo = $this->getDataInfoByPatient($dataInfoEvaluation);

            if($dataInfo) {
                $this->updateDataInfoEvaluation($dataInfoEvaluation);
            }else {
                $this->insertDataInfoEvaluation($dataInfoEvaluation);
            }

        }

    }

    private function initProcedure($patient) {

        $date = \DateTime::createFromFormat('d/m/Y H:i:s', $patient['appointmentDate'], new \DateTimeZone('America/Sao_Paulo'));
        
        $row = array(
            'CDFILIAL' => $patient['unit'],
            'CDPACIENTE' => $patient['id'],
            'DTCONSPACI' => $date->format('d-m-Y H:i:s'),
            'IDADEANO' => $patient['yearAndMonth'][0],
            'IDADEMES' => $patient['yearAndMonth'][1],
            'CDAVALIA' => $patient['avaliations'],
            'IDIOMA' => strval('P')
        );

        $sth = $this->CALCULA_METODOS_ANT($row);

    }

    public function CALCULA_METODOS_ANT($row) {
        
        $procedure = new StoredProcedure($this->connection, 'CALCULA_METODOS_ANT');
        $procedure->addParam(new Param('CDFILIAL',    Param::PARAM_INPUT, $row['CDFILIAL']));
        $procedure->addParam(new Param('CDPACIENTE',  Param::PARAM_INPUT, $row['CDPACIENTE']));
        $procedure->addParam(new Param('DTCONSPACI',  Param::PARAM_INPUT, $row['DTCONSPACI']));
        $procedure->addParam(new Param('IDADEANO',    Param::PARAM_INPUT, $row['IDADEANO']));
        $procedure->addParam(new Param('IDADEMES',    Param::PARAM_INPUT, $row['IDADEMES']));
        $procedure->addParam(new Param('CDAVALIA',    Param::PARAM_INPUT, $row['CDAVALIA']));
        $procedure->addParam(new Param('IDIOMA',      Param::PARAM_INPUT, $row['IDIOMA']));
        $procedure->execute();
        return true;
    }

    public function getResult($patient) {
        $erroToCalc = array(
            'ANTHROPOMETRIC' => '',
            'ERROR' => ''
        );
        try {
            $this->initProcedure($patient);
    
            $sth = $this->connection->prepare(self::RESULT_EVALUATION);
            $sth->execute(array(
                'CDPACIENTE' => $patient['id'],
                'CDFILIAL' => $patient['unit'],
                'DTCONSPACI' => $patient['appointmentDate']
            ));
    
            return $sth->fetchAll(\PDO::FETCH_ASSOC);
        } catch (\Exception $e) {
            $erroToCalc['ANTHROPOMETRIC'] = $patient;
            $erroToCalc['ERROR'] = $e->getMessage().' Error Code: '.$e->getCode();
            $erroToCalc['GENERIC_MESSAGE'] = "Erro ao calcular métodos da avaliação antropométrica!";
            return $erroToCalc;
        }

    }
    
    private function sanitazeNumber($stringToSanitaze){
        
        $stringToSanitaze = preg_replace('/[.]/', ',', $stringToSanitaze);
        $stringToSanitaze = preg_replace('/[^0-9,]/', '', $stringToSanitaze);
        return $stringToSanitaze;
    }
    
}