<?php
/**
 * Created by PhpStorm.
 * User: francismararaujo
 * Date: 23/10/14
 * Time: 17:07
 */

namespace TecDiet\Services;

use Doctrine\ORM\EntityManager;
use Util\Validation;
use TecDiet\Util\CRUD;

class ControlFeedingsService implements Queries {

    protected $entityManager;
    protected $connection;
    protected $crud;
    
    public function __construct(EntityManager $entityManager, CRUD $crud) {

        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
        $this->crud = $crud;
    }

    private function getSequential($controlFeeding) {
        $sth = $this->connection->prepare(self::GET_NVL_SEQUENTIAL_SERVINTE);
        $sth->execute(array(
            'PATIENT' => $controlFeeding['id'],
            'UNIT' => $controlFeeding['unit'],
            'SERVICE_DATE' => $controlFeeding['serviceDate']
        ));
        return $sth->fetch(\PDO::FETCH_NUM);
    }

    private function insertControlFeedings($controlFeeding) {
        
        $sequential = $this->getSequential($controlFeeding);

        $sth = $this->connection->prepare(self::INSERT_SERVINTE);

        if(!$controlFeeding['dietGroup']) {
            $controlFeeding['dietGroup'] = $controlFeeding['dietId'];
        }
        
        $stpaciacom;
        if(isset($controlFeeding['solicitante'])){
            $stpaciacom = $this->convertRequesterStringToNumber($controlFeeding['solicitante']);
        } else {
            $stpaciacom = 0;
        }
        
        $controlFeeding['stpaciacom'] = $stpaciacom;
        $controlFeeding['NRACOMPANA'] = 0;
        $controlFeeding['IDIMPRSERV'] = 'S';
        $controlFeeding['IDTIPOSERINT'] = NULL;
        $controlFeeding['IDRESTSERIN'] = NULL;
        $controlFeeding['DSSERVINTE'] = NULL;
        $controlFeeding['NRSERVPESO'] = NULL;
        $controlFeeding['sequential'] = str_pad($sequential[0], 2, "0", STR_PAD_LEFT);
        
        $sth->bindParam(':CDPACIENTE', $controlFeeding['id']);
        $sth->bindParam(':DTSERVINTE', $controlFeeding['serviceDate']);
        $sth->bindParam(':NRSEQUSERV', $controlFeeding['sequential']);
        $sth->bindParam(':CDFILIAL', $controlFeeding['unit']);
        $sth->bindParam(':CDSERVFILI', $controlFeeding['cdservfili']);
        $sth->bindParam(':CDDIETPAD', $controlFeeding['dietId']);
        $sth->bindParam(':NRACOMPANA', $controlFeeding['NRACOMPANA']);
        $sth->bindParam(':STPACIACOM', $controlFeeding['stpaciacom']);
        $sth->bindParam(':IDIMPRSERV', $controlFeeding['IDIMPRSERV']);
        $sth->bindParam(':IDTIPOSERINT', $controlFeeding['IDTIPOSERINT']);
        $sth->bindParam(':IDRESTSERIN', $controlFeeding['IDRESTSERIN']);
        $sth->bindParam(':CDNUTRICIO', $controlFeeding['nutritionist']);
        $sth->bindParam(':CDDIETGRUPSI', $controlFeeding['dietGroup']);
        $sth->bindParam(':DTINTERNA', $controlFeeding['hospitalization']);
        $sth->bindParam(':DSSERVINTE', $controlFeeding['DSSERVINTE']);
        $sth->bindParam(':NRSERVPESO', $controlFeeding['NRSERVPESO']);
        $sth->bindParam(':DTINCLUSAO', $controlFeeding['DTINCLUSAO']);
        
        $sth->execute();
        return $controlFeeding['sequential'];
    }
    
    private function convertRequesterStringToNumber($requesterString){
        
        if($requesterString == "P") {
            return 0;
        } else if($requesterString == "A") {
            return 1;
        }
        
    }

    public function save($controlFeedings) {
        $erroToSaveControlFeeding = array(
            'CONTROL_FEEDING' => array(),
            'ERROR' => array(),
            'OPERATION' => array(),
            'SEQUENTIAL' => ''
        );
        foreach ($controlFeedings as $controlFeeding) {

            if($controlFeeding['__is_new'] === true) {
                $this->connection->beginTransaction();
                try{
                    $sequential = $this->insertControlFeedings($controlFeeding);
                	$this->connection->commit();
                    $erroToSaveControlFeeding['SEQUENTIAL'] = $sequential;
                } catch (\Exception $e) {
                    $this->connection->rollback();
                    array_push($erroToSaveControlFeeding['CONTROL_FEEDING'], $controlFeeding);
                    array_push($erroToSaveControlFeeding['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                    array_push($erroToSaveControlFeeding['OPERATION'], 'INSERT');
                }
            } else {
                $this->updateControlFeedings($controlFeeding);
            }

        }

        return $erroToSaveControlFeeding;
    }
    
    public function delete($controlFeedings) {
        $tableName = 'SERVINTE';
        $erroToDeleteControlFeeding = array(
            'CONTROL_FEEDING' => array(),
            'ERROR' => array(),
            'OPERATION' => 'DELETE'
        );
        foreach($controlFeedings as $controlFeeding){
            $controlFeeding['cdpaciente'] = $controlFeeding['id'];
            $controlFeeding['nrsequserv'] = $controlFeeding['sequential'];
            $controlFeeding['dtservinte'] = $controlFeeding['serviceDate'];
            $controlFeeding['cdfilial'] = $controlFeeding['unit'];
            try{
                $this->crud->delete($tableName, $controlFeeding);
            } catch(\Exception $e){
                array_push($erroToDeleteControlFeeding['CONTROL_FEEDING'], $controlFeeding);
                array_push($erroToDeleteControlFeeding['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
            }
        }

        return $erroToDeleteControlFeeding;
    }
}