<?php
/**
 * Created by PhpStorm.
 * User: francismararaujo
 * Date: 23/10/14
 * Time: 17:07
 */

namespace TecDiet\Services;

use Doctrine\ORM\EntityManager;
use Util\Validation;

class PrescriptionsService implements Queries {

    protected $entityManager;
    protected $connection;

    public function __construct(EntityManager $entityManager) {

        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();

    }

    private function  updateCurrentBed($prescription) {
        
        $prescription['status'] = 'S';
        
        $sth = $this->connection->prepare(self::UPDATE_PRESCRIPTION_BED);

        $sth->bindParam(':BED', $prescription['bed']);
        $sth->bindParam(':UNIT', $prescription['unit']);
        $sth->bindParam(':STATUS', $prescription['status']);

        $sth->execute();

    }

    private function updateOldBed($prescription) {
        
        $prescription['status'] = 'N';

        $sth = $this->connection->prepare(self::UPDATE_PRESCRIPTION_BED);

        $sth->bindParam(':BED', $prescription['oldBed']);
        $sth->bindParam(':UNIT', $prescription['unit']);
        $sth->bindParam(':STATUS', $prescription['status']);

        $sth->execute();

    }

    private function updateBed($prescription) {

        if($prescription['bed'] !== $prescription['oldBed']) {

            $this->updateCurrentBed($prescription);
            $this->updateOldBed($prescription);

        }

    }

    private function updateHospitalization($prescription) {

        $sth = $this->connection->prepare(self::UPDATE_PRESCRIPTION_HOSPITALIZATION);

        $sth->bindParam(':BED', $prescription['bed']);
        $sth->bindParam(':PATIENT', $prescription['id']);
        $sth->bindParam(':UNITS', $prescription['unit']);

        $sth->execute();

    }

    private function updateDieteticPrescription($prescription) {

        $sth = $this->connection->prepare(self::UPDATE_DIETETIC_PRESCRIPTION);

        $sth->bindParam(':CDDIETPAD', $prescription['dietId']);
        $sth->bindParam(':DSPRESDIET', $prescription['prescription']);
        $sth->bindParam(':DSDADOCLIN', $prescription['clinical']);
        $sth->bindParam(':DSPRESMEDDIE', $prescription['medicalPrescription']);
        $sth->bindParam(':CDNIASNUTR', $prescription['nutritionalStatusId']);
        $sth->bindParam(':OLD_CDPACIENTE', $prescription['id']);
        $sth->bindParam(':OLD_DTPRESDIET', $prescription['date']);
        $sth->bindParam(':OLD_CDDIETPAD', $prescription['oldDietId']);
        $sth->bindParam(':OLD_CDFILIAL', $prescription['unit']);

        $sth->execute();

    }

    private function insertDieteticPrescription($prescription) {

        $sth = $this->connection->prepare(self::INSERT_DIETETIC_PRESCRIPTION);

        $sth->execute(array(
            'CDPACIENTE' => $prescription['id'],
            'DTPRESDIET' => $prescription['date'],
            'CDDIETPAD' => $prescription['dietId'],
            'DSPRESDIET' => $prescription['prescription'],
            'DSDADOCLIN' => $prescription['clinical'],
            'DSPRESMEDDIE' => $prescription['medicalPrescription'],
            'CDFILIAL' => $prescription['unit'],
            'CDNIASNUTR' => $prescription['nutritionalStatusId'],
            'CDLEITO' => $prescription['bed']
        ));

    }

    public function save($prescriptions) {
        $erroToSaveDietPrescription = array(
            'PRESCRIPTION' => array(),
            'ERROR' => array(),
            'OPERATION' => array()
        );
        foreach ($prescriptions as $prescription) {
            
            if(isset($prescription['__is_new']) && $prescription['__is_new'] == true) {
                try {
                    $this->insertDieteticPrescription($prescription);
                } catch (\Exception $e) {
                    array_push($erroToSaveDietPrescription['PRESCRIPTION'], $prescription);
                    array_push($erroToSaveDietPrescription['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                    array_push($erroToSaveDietPrescription['OPERATION'], 'INSERT');
                }
            } else {
                try {
                    $this->updateDieteticPrescription($prescription);
                } catch (\Exception $e) {
                    array_push($erroToSaveDietPrescription['PRESCRIPTION'], $prescription);
                    array_push($erroToSaveDietPrescription['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                    array_push($erroToSaveDietPrescription['OPERATION'], 'UPDATE');
                }
            }
        }
        return $erroToSaveDietPrescription;
    }
}