<?php
/**
 * Created by PhpStorm.
 * User: francismararaujo
 * Date: 23/10/14
 * Time: 17:07
 */

namespace TecDiet\Services;

use Doctrine\ORM\EntityManager;
use TecDiet\Util\Validation;
use TecDiet\Util\CRUD;


class ProfilesService implements Queries {

    protected $entityManager;
    protected $connection;
    protected $crud;
    public function __construct(EntityManager $entityManager, CRUD $crud) {
        $this->crud = $crud;
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();

    }

    private function getEatingHabits($patient, $unit) {

        $eatingHabits = $this->connection->prepare(self::EATING_HABITS);

        $eatingHabits->execute(
            array(
                'PATIENT' => $patient,
                'UNITS' => $unit
            )
        );

        return $eatingHabits->fetch(\PDO::FETCH_ASSOC);

    }

    private function getFoodIntolerance($patient, $unit) {

        $foodIntolerance = $this->connection->prepare(self::FOOD_INTOLERANCE);

        $foodIntolerance->execute(
            array(
                'PATIENT' => $patient,
                'UNITS' => $unit
            )
        );

        return $foodIntolerance->fetch(\PDO::FETCH_ASSOC);

    }

    private function updateFoodIntolerance($profile) {

        $foodIntolerance = $this->getFoodIntolerance($profile['id'], $profile['unit']);
        
        if(!$foodIntolerance) {
            $sth = $this->connection->prepare(self::INSERT_FOOD_INTOLERANCE);
        } else {
            $sth = $this->connection->prepare(self::UPDATE_FOOD_INTOLERANCE);
        }
        $sth->execute(
            array(
                "PATIENT" => $profile['id'],
                "UNITS" => $profile['unit'],
                "FOODINTOLERANCE" => $profile['foodIntolerance'],
            )
        );

    }

    private function updateEatingHabits($profile) {

        $eatingHabits = $this->getEatingHabits($profile['id'], $profile['unit']);
        
        if(!$eatingHabits) {
            $sth = $this->connection->prepare(self::INSERT_EATING_HABITS);
        } else {
            $sth = $this->connection->prepare(self::UPDATE_EATING_HABITS);
        }

        $sth->execute(
            array(
                "PATIENT" => $profile['id'],
                "UNITS" => $profile['unit'],
                "EATINGHABITS" => $profile['eatingHabits'],
                "CONFIDENTIALITY" => $profile['confidentiality'],
                "OBSERVATION" => $profile['observation'],
            )
        );

    }

    private function updateLastVisit($lastVisit, $patient, $unit) {
        $sth = $this->connection->prepare(self::UPDATE_LAST_VISIT);
        $sth->bindParam(':DTULTATU', $lastVisit);
        $sth->bindParam(':CDPACIENTE', $patient);
        $sth->bindParam(':CDFILIAL', $unit);
        $sth->execute();
    }
    
    private function insertFoodIntoleranceGroup($profile) {
        
        $sth = $this->connection->prepare(self::INSERT_FOOD_INTOLERANCE_GROUP);
        $sth->bindParam(':CDFILIAL', $profile['unit']);
        $sth->bindParam(':CDPACIENTE', $profile['id']);
        $sth->bindParam(':CDGRUPPROD', $profile['cdgrupprod']);
        $sth->bindParam(':CDSUBGRPROD', $profile['cdsubgrprod']);
        $sth->execute();
    }

    public function save($profiles) {
        $erroToSaveProfile = array(
            'PROFILE' => array(),
            'ERROR' => array(),
            'OPERATION' => array()
        );
        $genericMessage = "Erro ao atualizar as descrições do perfil alimentar";
        foreach($profiles as $profile){
            if(isset($profile["fieldFoodGroup"])){
                try{
                    $this->insertFoodIntoleranceGroup($profile);
                } catch(\Exception $e){
                    $profile['SUBCATEGORY'] = 'FOOD_INTOLERANCE_GROUP';
                    $profile['SUBCATEGORYNAME'] = 'Grupos de Intolerância Alimentar';
                    array_push($erroToSaveProfile['PROFILE'], $profile);
                    array_push($erroToSaveProfile['PROFILE'], $e->getMessage().' Error Code: '.$e->getCode());
                    array_push($erroToSaveProfile['OPERATION'], 'INSERT');
                }
            }
            if(isset($profile['fieldProfileEdit'])){
                try {
                    $this->updateEatingHabits($profile);
                    $this->updateFoodIntolerance($profile);
                } catch (\Exception $e) {
                    $profile['GENERIC_MESSAGE'] = $genericMessage;
                    array_push($erroToSaveProfile['PROFILE'], $profile);
                    array_push($erroToSaveProfile['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
                    array_push($erroToSaveProfile['OPERATION'], 'UPDATE');
                }
            }
            $this->updateLastVisit($profile['lastVisit'], $profile['id'], $profile['unit']);
        }
        return $erroToSaveProfile;
    }
    
    public function delete($profilesDelete) {
        $tableName = 'INTOALIM';
        $erroToDeleteFoodGroup = array(
            'PROFILE' => array(),
            'ERROR' => array(),
            'OPERATION' => 'DELETE'
        );
        foreach($profilesDelete as $profileDelete){
            try {
                if(!isset($profileDelete["cdfilial"])){
                    $unit = $profileDelete['unit'];
                    $profileDelete['cdfilial'] = $unit;
                }
                if(isset($profileDelete["fieldFoodGroup"])){
                        $this->crud->delete($tableName, $profileDelete);
                        $this->updateLastVisit($profileDelete['lastVisit'], $profileDelete['id'], $profileDelete['unit']);
                }
            } catch (\Exception $e) {
                $profileDelete['GENERIC_MESSAGE'] = 'Erro ao remover grupos de intolerância alimentar do perfil alimentar';
                array_push($erroToDeleteFoodGroup['PROFILE'], $profileDelete);
                array_push($erroToDeleteFoodGroup['ERROR'], $e->getMessage().' Error Code: '.$e->getCode());
            }
        }
        return $erroToDeleteFoodGroup;
    }
}