module.exports = function(grunt) {
    var srcFiles = {
        js: [
            "js/libraries/*.js",
            "js/repositories/*.js",
            "js/util/*.js",
            "js/services/*.js",
            "js/controllers/*.js"
            ],
        css: [
            "css/*.css", 
            "dist/*.css", 
            "bower_components/red-ui/dist/rui.css", 
            "bower_components/red-ui-header/dist/rui-header.css", 
            "bower_components/red-ui-fields/dist/rui-fields.css", 
            "bower_components/red-ui-sidenav/dist/rui-sidenav.css",
            "tecdiet_templates/css/*.css"
            ],
        libs: [
			"./bower_components/zh-preferences/src/ZhPreferences.js",
            "./bower_components/zh-customizations-service/dist/zhCustomizationsService.js", 
            "./bower_components/zh-cache-bypass/src/CacheBypass.js", 
            "./bower_components/red-ui/dist/rui.js", 
            "./bower_components/red-ui-header/dist/rui-header.js", 
            "./bower_components/red-ui-fields/dist/rui-fields.js", 
            "./bower_components/red-ui-sidenav/dist/rui-sidenav.js",
            "./tecdiet_templates/js/*.js"
            ],
		environment: [
			"./js/environment.js"
		],
		ZeedhiConfig: [
			"./config/zeedhiConfig.js"
		]
    };
    var typeOfScope = ['Dev', 'Merge', 'Test'];
    var jshintOptions = {
        "globals": {
            "Configuration": true,
			"moment": true,
			"$": true,
			"console": true,
			"angular": true,
			"$rootScope": true,
			"getLocalVar": true,
			"GeneralMethods": true,
			"setLocalVar": true,
			"metaDataFactory": true,
			"module": true
        },
        "unused": true,
        "undef": true,
        "browser": true,
        "bitwise": true,
        "curly": true,
        "eqeqeq": true,
        "freeze": true,
        "loopfunc": false,
        "debug": true,
        "noempty": true,
        "nonew": true,
        "trailing": true,
        reporter: require('jshint-html-reporter'),
        reporterOutput: './logs/dev.html'
    };

    function setSpecific(currScope) {
        switch (true) {
            case (currScope === typeOfScope[1]): //merge
                jshintOptions.devel = true;
                break;
            case (currScope === typeOfScope[2]): //test
                jshintOptions.devel = true;
                break;
            default: //dev
                jshintOptions.devel = false;
        }
        return jshintOptions;
    }
    grunt.initConfig({
        less: {
            development: {
                options: {
                    compress: true,
                    modifyVars: {
                        splashColor: '#FFFFFF',
                        primaryColor: '#425056',
                        accentColor: '#000000',
                        lightPrimaryColor: '#FFCDD2',
                        textIconsColor: '#FFFFFF',
                        primaryText: '#212121',
                        secondaryText: '#757575',
                        dividerColor: '#BDBDBD',
                        errorColor: "#DD2C00",
                        fontFamily: 'Roboto, san serif',
                        defaultFontSize: '1rem',
                        cartRowSelected: '#AAAAAA',
                        darkPrimaryColor: '#242c2f',
                        listRowBgColor: '#FFFFFF'
                    }
                },
                files: {
                    'bower_components/red-ui/dist/rui.css': "bower_components/red-ui/src/assets/css/**/*.less",
                    'bower_components/red-ui-header/dist/rui-header.css': "bower_components/red-ui-header/src/assets/css/**/*.less",
                    'bower_components/red-ui-fields/dist/rui-fields.css': "bower_components/red-ui-fields/src/assets/css/**/*.less",
                    'bower_components/red-ui-sidenav/dist/rui-sidenav.css': "bower_components/red-ui-sidenav/src/assets/css/**/*.less",
                    'css/custom.css': "css/custom.less"
                }
            }
        },
        jshint: {
            merge: {
                src: ['Gruntfile.js', srcFiles.js],
                "options": setSpecific(typeOfScope[1])
            },
            dev: {
                src: ['Gruntfile.js', srcFiles.js],
                "options": setSpecific(typeOfScope[0])
            }
        },
        concat: {
            css: {
                options: {
                    process: function(src, filepath) {
                        return '/* FILE: ' + filepath + ' */\n' + src + '\n';
                    }
                },
                src: srcFiles.css,
                dest: 'dist/app.css'
            },
            js: {
                options: {
                    process: function(src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: srcFiles.js,
                dest: 'dist/app.js'
            },
            libs: {
                options: {
                    process: function(src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: srcFiles.libs,
                dest: 'dist/libs.js'
            }
        },
        jsbeautifier: {
            final: {
                src: ['Gruntfile.js', srcFiles.js],
                options: {
                    js: {
                        maxPreserveNewlines: 0,
                        preserveNewlines: false
                    }
                }
            },
            dev: {
                src: ['Gruntfile.js', srcFiles.js],
                options: {
                    js: {
                        maxPreserveNewlines: 2
                    }
                }
            }
        },
        uglify: {
            options: {
                mangle: false,
                compress: false,
                report: 'min',
                // the banner is inserted at the top of the output
                banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
            },
            dist: {
                files: {
                    'dist/app.min.js': srcFiles.js,
                    'dist/libs.min.js': srcFiles.libs,
                    "js/environment.min.js": srcFiles.environment,
                    "config/zeedhiConfig.min.js": srcFiles.ZeedhiConfig
                }
            }
        },
        cssmin: {
            dist: {
                files: {
					"dist/app.min.css"           : srcFiles.css,
					"bower_components/zeedhi-frontend/assets/fonts/font-zeedhi/stylesheet.min.css"    : "bower_components/zeedhi-frontend/assets/fonts/font-zeedhi/stylesheet.css",
					"bower_components/zeedhi-frontend/assets/css/styles/styles-tecdiet.min.css"       : "bower_components/zeedhi-frontend/assets/css/styles/styles-tecdiet.css",
					"bower_components/zh-preferences/dist/style/style.min.css"                        : "bower_components/zh-preferences/dist/style/style.css",
					"bower_components/zh-preferences/dist/style/colors/styles-tecdiet.min.css"        : "bower_components/zh-preferences/dist/style/colors/styles-tecdiet.css",
					"bower_components/zh-select-autocomplete/dist/style/colors/styles-tecdiet.min.css": "bower_components/zh-select-autocomplete/dist/style/colors/styles-tecdiet.css",
					"bower_components/zh-customizations-service/dist/style/colors/styles-tecdiet.min.css": "bower_components/zh-customizations-service/dist/style/colors/styles-tecdiet.css",
					"bower_components/zh-customizations-service/dist/style/style.min.css": "bower_components/zh-customizations-service/dist/style/style.css",
					"bower_components/zh-rich-text/dist/zh-rich-text.min.css": "bower_components/zh-rich-text/dist/zh-rich-text.css"
				}
            }
        },
        watch: {
            js: {
                files: srcFiles.js,
                tasks: ['jsbeautifier:dev', "jshint", 'concat:js', 'buildEnvironment']
            },
            css: {
                files: srcFiles.css,
                tasks: ['concat:css']
            }
        },
        // NÂO COLOCAR PROPRIEDADE PARA SUBSTIUIR IDS
        zhIdGenerator: {
            options: {
                jsonPath: ["./json"]
            }
        },
        buildEnvironment: {
            development: {
                configFile: "./config/development.json"
            },
            deploy: {
                configFile: "./config/deploy.json"
            },
            test: {
                configFile: "./config/test.json"
            },
            srcFolders: ["../backend/config/", "./config", "./"]
        }
    });
    grunt.loadNpmTasks('grunt-contrib-jshint');
    grunt.loadNpmTasks('grunt-contrib-uglify');
    grunt.loadNpmTasks('grunt-contrib-cssmin');
    grunt.loadNpmTasks('grunt-contrib-concat');
    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks("grunt-jsbeautifier");
    grunt.loadNpmTasks('grunt-zh-id-generator');
    grunt.loadNpmTasks('grunt-contrib-less');
    grunt.loadNpmTasks('grunt-zh-environment-builder');
    grunt.loadNpmTasks("grunt-notify");

    // Default task.
    grunt.registerTask('default', ['less', 'jsbeautifier:dev', 'concat', 'buildEnvironment', 'cssmin', 'uglify', 'jshint:dev']);
    // Other tasks.
    grunt.registerTask('devCoPilot', ['watch', 'notify']);
    grunt.registerTask('merge', ['jsbeautifier:final', 'jshint:merge', 'concat']);
    grunt.registerTask('dev', ['jsbeautifier:dev', 'jshint:dev', 'concat', 'buildEnvironment', 'cssmin']);
    grunt.registerTask('prod', ['jsbeautifier:final', 'concat', 'buildEnvironment', 'cssmin']);
    grunt.registerTask('test', ['jsbeautifier:final', 'concat', 'buildEnvironment', 'cssmin']);
};