var PrescriptionController = function(Validation, ScreenService, Environment, PrescriptionService) {
    var containerMain;
    var widgetStripe;
    var widgetMain;
    var _verify = function(widget) {
        widget.reload().then(function() {
            widget.dataSource.data.forEach(function(prescription) {
                if (Date.parse(prescription.date) < Date.parse(Validation.now())) {
                    widget.dataSource.data.push(prescription);
                }
            });
            widget.moveToFirst();
        });
    };
    this.openHistory = function(widget) {
        widget.editing = true;
        ScreenService.openSwipe(widget);
    };
    this.onEnter = function(widget) {
        _verify(widget);
    };
    var _initStripe = function() {
        widgetStripe.currentRow = Environment.getCurrentPatient();
    };
    var _initContainer = function(container) {
        containerMain = container;
        widgetStripe = container.getWidget('WidgetStripe');
        widgetMain = container.getWidget('WidgetPrescriptionsHistoryMain');
    };
    this.containerAfterinit = function(container) {
        _initContainer(container);
        _initStripe();
    };
    var _validateFields = function(fields) {
        if (!fields.bed) {
            throw "Informe o leito.";
        }
        if (!fields.date) {
            throw "Informe a data da prescrição.";
        }
        if (!fields.dietId) {
            throw "Informe a dieta.";
        }
        if (!fields.nutritionalStatusId) {
            throw "Informe o nível nutricional.";
        }
    };
    this.edit = function(widget) {
        var prev = widget.container.getWidget('WidgetPrescription').currentRow;
        if (prev !== undefined) {
            widget.widgets[0].currentRow = widget.container.getWidget('WidgetPrescription').currentRow;
            widget.widgets[0].template = "widget/form.html";
            widget.widgets[0].edit();
            widget.widgets[0].getField('bed').readOnly = true;
            widget.widgets[0].getField('date').readOnly = true;
            widget.widgets[0].getField('dietName').readOnly = true;
            widget.widgets[0].label = ScreenService.i18n('Editar Prescrição');
            ScreenService.openSwipe(widget);
        } else {
            var msg = ScreenService.i18n('Não há itens a serem editados.');
            ScreenService.showMessage(msg);
        }
    };
    this.add = function(widget) {
        widget.widgets[0].getField('date').readOnly = false;
        widget.widgets[0].getField('dietName').readOnly = false;
        widget.widgets[0].newRow();
        widget.widgets[0].label = ScreenService.i18n('Adicionar Prescrição');
        widget.widgets[0].getField('bed').value(Environment.getCurrentPatient().bed);
        ScreenService.openSwipe(widget);
    };
    this.confirm = function(widget) {
        var WidgetPrescription = widget.container.getWidget('WidgetPrescription');
        try {
            _validateFields(widget.currentRow);
            widget.currentRow.changed = true;
            widget.currentRow.id = Environment.getLocalVar('__patient');
            widget.currentRow.unit = Environment.getLocalVar('__CDFILIAL');
            PrescriptionService.save(widget.currentRow).then(function() {
                Environment.lastPatientVisit();
                _verify(widget.container.getWidget('WidgetPrescription'));
                ScreenService.closeSwipe();
                // ScreenService.showMessage('Salvo com sucesso.');
                WidgetPrescription.activate();
            });
        } catch (exceptionMessage) {
            ScreenService.showMessage(exceptionMessage);
        }
    };
    // HISTORY
    this.openSwipe = function(widget) {
        widget.widgets[0].widgets[0].currentRow = widget.currentRow;
        widget.widgets[0].widgets[0].view();
        widget.widgets[0].widgets[0].activate();
        ScreenService.openSwipe(widget.widgets[0]);
    };
    this.cancel = function(widget) {
        var WidgetPrescription = widget.container.getWidget('WidgetPrescription');
        ScreenService.closeSwipe().then(function() {
            WidgetPrescription.activate();
        });
    };
};
Configuration(function(ContextRegister) {
    ContextRegister.register('PrescriptionController', PrescriptionController);
});
