var CRUD = function(localStorageEngine, ZHPromise) {
    var self = this;
    this.init = function(dataSourceName) {
        this.dataSourceName = dataSourceName;
        return this;
    };
    this.find = function(filter, findDeleted, dataSourceName) {
        if (findDeleted === undefined || findDeleted === null) {
            findDeleted = false;
        }
        if (dataSourceName) {
            this.dataSourceName = dataSourceName;
        }
        var defer = ZHPromise.defer();
        localStorageEngine.getDataSource(this.dataSourceName, filter).then(function success(rows) {
            defer.resolve(rows.filter(function(row) {
                return (findDeleted || !row.localChanges || !row.localChanges.remove);
            }));
        }, function error(e) {
            defer.reject(e);
        });
        return defer.promise;
    };
    this.saveRow = function(row, dataSourceName) {
        if (dataSourceName) {
            this.dataSourceName = dataSourceName;
        }
        if (row.localChanges && row.localChanges.remove) {
            delete row.localChanges.remove;
        }
        return localStorageEngine.save(this.dataSourceName, row);
    };
    this.saveDataset = function(dataset, dataSourceName) {
        var promises = [];
        for (var i = 0; i < dataset.length; i++) {
            var rowToSave = dataset[i];
            promises.push(self.saveRow(rowToSave, dataSourceName));
        }
        return ZHPromise.all(promises);
    };
    this.removeRow = function(row, dataSourceName) {
        if (dataSourceName) {
            this.dataSourceName = dataSourceName;
        }
        row.localChanges = row.localChanges || {};
        row.localChanges.remove = true;
        row.changed = true;
        return localStorageEngine.save(this.dataSourceName, row);
    };
    this.removeDataset = function(dataset, dataSourceName) {
        var promises = [];
        for (var i = 0; i < dataset.length; i++) {
            var rowToRemove = dataset[i];
            promises.push(self.removeRow(rowToRemove, dataSourceName));
        }
        return ZHPromise.all(promises);
    };
};
Configuration(function(ContextRegister) {
    ContextRegister.register('CRUD', CRUD);
});
