function MNListController ($scope, $timeout, ScreenService, eventAggregator, templateManager, localStorageEngine, restEngine, ZHPromise, ApplicationContext) {
    var container = $("#mn-list-container");
    var widget = $scope.widget;
    var keys = ['/Profiles', '/Units', '/Patients', '/Halls', '/Prescriptions', '/NormalFeedings', '/BioEvaluations', '/BedsPrescriptions', '/NutritionalStatus', '/Diets', '/DietsServices', '/Storeys', '/Beds', '/Appointments', '/HealthInsurance', '/MedicalTeams', '/Examinations', '/AnthEvaluationMethods', '/AnthEvaluationGeneral', '/DataTypeEvaluations', '/DataEvaluations', '/ControlFeedings', '/Services', '/LactaryPlates', '/Plates', '/LactaryFeedings', '/EnteralFeedings', '/HallsDistincts', '/PlatesByUnit', '/PlatesPla', '/FoodIntoleranceGroup', '/AllGroupSubgroup', '/PatientIntoRestriction', '/EnteralPlates', '/EnteralProducts', '/OralServices', '/LactaryServices', '/EnteralServices'];
    var intoleranciaCrud = ApplicationContext.CRUD.init("/FoodIntoleranceGroup");
    var controlFeedingCrud = ApplicationContext.CRUD.init("/ControlFeedings");
    var lactaryFeedingCrud = ApplicationContext.CRUD.init("/LactaryFeedings");
    var enteralFeedingCrud = ApplicationContext.CRUD.init("/EnteralFeedings");

    this.sendPatientUpdate = function(patient, situation) {
        var widget = $scope.widget;        
        var messages = ['Deseja confirmar <b>alta</b> deste paciente selecionado?', 'Deseja confirmar <b>óbito</b> do paciente selecionado?'];
        ScreenService.confirmMessage(messages[situation], 'question', function() {
            eventAggregator.publish("sendPatientUpdate", {'patient': [patient], 'situation': situation,'widget': widget});
        });
    };
    
    eventAggregator.subscribe('openPatientDetails', function(patient){
        openPatientDetails(patient);
    });
    this.openPatientDetails = function(patient){
        openPatientDetails(patient);
    };
    function openPatientDetails (patient) {
        $scope.widget.container.getWidget('WidgetMain').curPatient = patient;
        var widgetPatient = widget.container.getWidget('WidgetDetailsPatient');
        widgetPatient.activate();
        widgetPatient.isVisible = true;
        widgetPatient.curPatient = patient;
        widgetPatient.currentScreen = 'personalData';
        widgetPatient.showBioqExamList = false;
        loadStorage(widgetPatient).then(function (dataSources) {
            widgetPatient.localDataSourceLoaded = dataSources;
            sizeWindow(); 
            addData(patient);
            setLocalVar('__patient', patient.id);
            setLocalVar('__CDFILIAL', patient.unit);
            
            var intoleranciaFilter = [{
                'name': 'cdpaciente',
                'operator': '=',
                'value': widgetPatient.curPatient.id
            }];
            intoleranciaCrud.find(intoleranciaFilter, false, "/FoodIntoleranceGroup").then(function(dataFoodGroup){
                widgetPatient.dataFoodGroup = angular.copy(dataFoodGroup);
            });

        });
    };
    
    function sizeWindow() {
        var pacientDetailsControlers = parseInt($('.pacientData ').css('height')) + parseInt($('.viewControlsContainer ').css('height')) - 30;
        $('.pacientDetailsContainer').css('height', 'calc(100% - ' + pacientDetailsControlers + 'px)');
    }
    
    function addData(patient) {
        var datasources = widget.container.getWidget('WidgetDetailsPatient').localDataSourceLoaded;
        var searchForId = patient.id;
        var dataToBeAnexed = [];
        keys.forEach(function (keyDirty) {
            var keyName = keyDirty.replace('/', '');

            if (keyName !== 'Patients' &&
                keyName !== 'NutritionalStatus' &&
                keyName !== 'AnthEvaluationGeneral' &&
                keyName !== 'NormalFeedingsNow' &&
                datasources[keyName] !== null) {

                datasources[keyName].forEach(function (feedInfo) {
                    if (feedInfo.id == searchForId) {
                        dataToBeAnexed.push(feedInfo);
                    }
                });

                patient[keyName] = dataToBeAnexed;
                dataToBeAnexed = [];
            }
        });
        addLocationDataToPrescription(patient);
    }
    
    function addLocationDataToPrescription(patient) {
        if (patient.Prescriptions) {
            patient.Prescriptions.forEach(function (pres, key) {
                var curBedID = pres.bed;
                widget.container.getWidget('WidgetDetailsPatient').localDataSourceLoaded.Beds.forEach(function (beditem) {
                    if (beditem.bed === curBedID) {
                        patient.Prescriptions[key].hall = beditem.hall;
                        patient.Prescriptions[key].storey = beditem.storey;
                    }
                });
            });
        }
    }
    
    function loadStorage(widget) {
        return ZHPromise.all(keys.map(function (key) {
            if(key === '/ControlFeedings'){
                return controlFeedingCrud.find(null,false,'/ControlFeedings').then(function(dataSource){
                    return dataSource;
                });
            }else if(key === '/LactaryFeedings'){
                return lactaryFeedingCrud.find(null,false,'/LactaryFeedings').then(function(dataSource){
                    return dataSource;
                });
            } else if(key === '/EnteralFeedings'){
                return enteralFeedingCrud.find(null, false, '/EnteralFeedings').then(function(dataSource){
                    return dataSource;
                });
            }else{
                return localforage.getItem(key).then(function (value) {
                    return JSON.parse(value);
                }.bind(this));
            }
        }.bind(this))).then(function (loadedDataSources) {
            var dataSources = {};
            keys.forEach(function (key, index) {
                var keyName = key.replace('/', '');
                if ((keyName == 'Appointments' || keyName == 'Prescriptions') && loadedDataSources[index]) {
                    loadedDataSources[index].filter(function (attr) {
                        attr.normalizedDate = attr.date;
                        attr.date = moment(attr.date, 'DD/MM/YYYY HH:mm:ss').format('YYYY-MM-DDTHH:mm:ss');
                    });
                }
                dataSources[keyName] = loadedDataSources[index] ? loadedDataSources[index] : []; 
            });
            return dataSources;
        });
    }
    this.addPatient = function(widget){
        var newPatientPopup = widget.container.getWidget('newPatientPopup');
        newPatientPopup.newRow();
        var ISSEQUENTIAL = getLocalVar("__ISSEQUENTIAL");
        if(!ISSEQUENTIAL){
            ScreenService.showMessage(ScreenService.i18n('Não existe parametrização definida nesta unidade para gerar código do paciente!'));
        }else{
            if(ISSEQUENTIAL === 'S'){
                newPatientPopup.getField("id").readOnly = true;
                restEngine.post('/Patients/patientSequential', {
                    "requestType": "Row",
                    "row": {
                        'unit': localStorageEngine.getLocalVar('__CDFILIAL'),
                        'isAjusta': getLocalVar("__ISAJUSTA")
                    }
                }).then(function(callback) {
                    newPatientPopup.currentRow.id = callback.dataset.patientSequential.CDPACIENTE;
                });
            
            }else{
                newPatientPopup.getField("id").readOnly = false;
            }
            ScreenService.openPopup(newPatientPopup);
            newPatientPopup.newRow();
            newPatientPopup.edit();
        }
    };
    
    this.addHospitalization = function(widget, patient){
        
        var newHospitalizationPopup = widget.container.getWidget('newHospitalizationPopup');
        newHospitalizationPopup.newRow();
        newHospitalizationPopup.edit();
        newHospitalizationPopup.getField('bed').dataSourceFilter =  [{
                "name": [{
                    "name": "bed",
                    "direction": "ASC"
                }],
                "operator": "ORDER_BY",
                "value": ""
            },
            {
                "name": "unit",
                "operator": "=",
                "value": getLocalVar('__CDFILIAL')
            }
        ];
        if(patient){
            setTimeout(function(){
                newHospitalizationPopup.getField('id').disabled = true;
                newHospitalizationPopup.currentRow.id = patient.id;
                newHospitalizationPopup.currentRow.idnamepaci = patient.idnamepaci;
                newHospitalizationPopup.getField('id').reload();
            }, 1000);
            newHospitalizationPopup.getField('id').readOnly = true;
        }else{
            newHospitalizationPopup.getField('id').disabled = false;
            newHospitalizationPopup.getField('id').readOnly = false;
        }
        newHospitalizationPopup.currentRow.hospitalizationDate = moment().format('YYYY-MM-DDTHH:mm');
        eventAggregator.publish("reloadAllFields", newHospitalizationPopup);
        ScreenService.openPopup(newHospitalizationPopup);
    };
    
    $scope.setPatientAge = function(fieldEdited) {
    	eventAggregator.publish("setPatientAge", fieldEdited);
    };

    this.infiniteScroll = function() {
        var scrollHeight = container[0].scrollHeight - 15;
        var scrollPosition = container.height() + container.scrollTop() - 15; //padding
        var previousScroll = previousScroll || scrollPosition;
        if (((scrollHeight - scrollPosition) / scrollHeight < 0.030) && previousScroll <= scrollPosition) {
            this.loadData(true);
        }
        previousScroll = scrollPosition;
    };

    /**
     * Retrieve content from list dataSource
     * @param  {Boolean} apply - Scope apply selector
     */
    this.loadData = function(apply) {
        if (apply) {
            $scope.$apply();
        }
        var widget = $scope.widget;
        this.nextPage(widget).then(function(callback) {
        	var data;
            if (callback && callback.dataset) {
                Object.keys(callback.dataset).forEach(function (datasetName) {
                    data = callback.dataset[datasetName];
                });
            } else {
                data = callback || [];
            }
            $scope.loadedAllData = data.length < widget.itemsPerPage;
            $scope.loadingData = false;
        }.bind(this));
    };
    
    this.nextPage = function(widget) {
        if (widget.dataSource) {
            return widget.dataSource.load(widget.getDataSourceFilter(), widget.dataSource.getNextPage(), widget.itemsPerPage);
        } else {
            return ZHPromise.when();
        }
    };
    
    this.searchPatient = function(widget){
        var search = angular.copy(widget.typeSearch);
        if(search){
            search = removerAcentos(search);
        }
        if(!widget.dataSourceFilter){
            widget.dataSourceFilter = [{
                name: "normalizedName",
                operator: "LIKE",
                value: "%"+search+"%"
            }];
        }else{
            var noDataFound = true;
            widget.dataSourceFilter.forEach(function(filter){
               if(filter.name === 'normalizedName'){
                   filter.value = "%"+search+"%";
                   noDataFound = false;
               } 
            });
            if(noDataFound){
                widget.dataSourceFilter.push({
                    name: "normalizedName",
                    operator: "LIKE",
                    value: "%"+search+"%"
                });
            }
        }
        widget.reload();
    };
    
    function removerAcentos(newStringComAcento) {
        var string = newStringComAcento;
        var mapaAcentosHex = {
            a: /[\xE0-\xE6]/g,
            A: /[\xC0-\xC6]/g,
            e: /[\xE8-\xEB]/g,
            E: /[\xC8-\xCB]/g,
            i: /[\xEC-\xEF]/g,
            I: /[\xCC-\xCF]/g,
            o: /[\xF2-\xF6]/g,
            O: /[\xD2-\xD6]/g,
            u: /[\xF9-\xFC]/g,
            U: /[\xD9-\xDC]/g,
            c: /\xE7/g,
            C: /\xC7/g,
            n: /\xF1/g,
            N: /\xD1/g
        };

        for (var letra in mapaAcentosHex) {
            var expressaoRegular = mapaAcentosHex[letra];
            string = string.replace(expressaoRegular, letra);
        }

        return string;
    }

    container.on("scroll", this.infiniteScroll.bind(this));
    
    $scope.formatAge = function(patient){
        if(patient){
            var anos = patient.yearAndMonth[0];
            var meses = patient.yearAndMonth[1];
            
            if(anos === "0"){
                return meses + ' ' + ScreenService.i18n('Mês(es)');
            }else if(meses === "0"){
                return anos + ' ' + ScreenService.i18n('Ano(s)');
            }else{
                return anos + ' ' + ScreenService.i18n('Ano(s)')  + ' ' +  ScreenService.i18n('e') + ' '+ meses  + ' ' + ScreenService.i18n('Mês(es)');
            }
        }
    };
}
